/* This file is part of MySQLConnectus by Sergio Decherchi

Copyright (C) 2004 Sergio Decherchi

This program is free software; you can redistribute it 
and/or modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
GNU General Public License for more details (gpl.txt).
You should have received a copy of the GNU General Public License 
along with this program; if not, write to the Free Software Foundation, 
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */

import javax.swing.plaf.*;
import javax.swing.border.*;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.text.html.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileView;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterJob;
import java.awt.print.PrinterException;
import java.io.*;
import java.util.*;
import java.net.*;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.*;

/**Main window of the program*/

public class MainFrame extends JFrame
{

//General variables
	
	//For memory optimization and runtime information
	final private Runtime	runtimeStatus=Runtime.getRuntime();		
	//Unique reused instance of access to the DBMS
	final private Axxess currentAccess=new Axxess();
	//OrederedData variable (see the class)	
	final private	OrderedData orderedData;
	//Common utility variables
	private		String	temp,user,password,currentQuery=null;
	private	String	queryText;
	//Current selected DB
	private String	currentDB="";
	//Used for compatibility with drawString method instead of \t
	private 	String	space="                        ";
	
//Theme variables
	
	//Strings rapresenting themes
	final private String windowsStyle="com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
	final private String javaStyle="javax.swing.plaf.metal.MetalLookAndFeel";
	final private String motifStyle	="com.sun.java.swing.plaf.motif.MotifLookAndFeel";
	//Variable for current theme	
	private	DefaultMetalTheme	theme=null;
	//Flag for theme validation
	private	boolean	styleOk	=true;	
	
//Main window variables
	
	//Internal frames title
	private	String	windowTitle="";
	//The reference for internal frames
	private	JDesktopPane	desktop=null;
	// Help on line window
	private HtmlHelpOnLine	hhol=null;
	//This reference variable rapresents the current instances of the tables
	private	Table tab,savedTable=null;
	//Index for the location of the query windows
	//when a new window is opened this is placed in a different position
	//respect the previuos one
	private int	winpos=0;
	//Instance for the window quering password/user
	private	PasswordChooser	dialog=null;
	//Current tree
	private Tree	currentTree=null;
	//Flag for tree
	private boolean showImmediatelyRelations=false;
	//Flag for dialog windows management
	private	boolean accessDone=false;			
	
//Variables for autoconfig settings
	
	private	boolean	nextSessionIsJavaStyle	=true;
	private	int		nextSessionJavaTheme	=1;
	private	String	nextSessionStyle		=null;
	private	String	nextSessionScript		=null;
	private	boolean	nextSessionUseLastLoadedScript=false;
	private	boolean	isAutoConfigEnabled		=true;	 	
	private	boolean	scriptSet=false;
	
/**Main constructor*/

	public MainFrame()
		
	{    	
		//Splash screen
		JWindow	splashScreen=null;
		JLabel	splashLabel=null;
		boolean	noSplash=false;
		//Screen information
		int screenWidth=0;
		int screenHeight=0;
		//Tools bar
		JToolBar	jtb=null;
	
	
		//Splash screen
		try
		{
		splashLabel = new JLabel(new ImageIcon(getClass().getResource("/Resources/Splash2.jpg")));
		splashScreen = new JWindow(MainFrame.this);
		splashScreen.getContentPane().add(splashLabel);
		splashScreen.pack();
		Rectangle screenRect = (MainFrame.this).getGraphicsConfiguration().getBounds();
		splashScreen.setLocation(
    	screenRect.x + screenRect.width/2 - splashScreen.getSize().width/2,
		screenRect.y + screenRect.height/2 - splashScreen.getSize().height/2);
		splashScreen.show();
		}
		catch (NullPointerException npe)
		{
		JOptionPane.showMessageDialog(null,"Can't find splash2.jpg","Info",0);								;
		noSplash=true;
		}
		orderedData=new OrderedData();
		//Setup main frame
		setTitle("MySQLConnectus 0.3");
		Toolkit kit 		= Toolkit.getDefaultToolkit();
		Dimension screenSize= kit.getScreenSize();
		screenWidth 		=((screenSize.width));
		screenHeight		=((screenSize.height)-25);
		setLocation(0,0);
		setSize(screenWidth,screenHeight);
		//Creation of the main window
		MainPanel panel=new MainPanel();
		desktop = new JDesktopPane();
		Container contentPane=getContentPane();
		jtb=new JToolBar();
		//Setting toolbar
		JButton writeqb	=	new JButton(new ImageIcon("Resources/palette_draw.gif"));
		JButton openb	=	new JButton(new ImageIcon("Resources/open.gif"));
		JButton writemb	=	new JButton(new ImageIcon("Resources/palette_draw.gif"));
		JButton openmb	=	new JButton(new ImageIcon("Resources/open.gif"));
		JButton loginb	=	new JButton(new ImageIcon("Resources/next.gif"));
		JButton scriptb	=	new JButton(new ImageIcon("Resources/script.gif"));
		JButton opentb	=	new JButton(new ImageIcon("Resources/open.gif"));
		JButton showtb	=	new JButton(new ImageIcon("Resources/showdata.gif"));
		JButton chdbb	=	new JButton(new ImageIcon("Resources/change.gif"));
		JButton showb	=	new JButton(new ImageIcon("Resources/showdata.gif"));
		JButton autob	=	new JButton(new ImageIcon("Resources/gear.gif"));
		JButton helpb	=	new JButton(new ImageIcon("Resources/HelpCenter.gif"));
		JButton exitb	=	new JButton(new ImageIcon("Resources/prev.gif"));
		openb.addActionListener(new Query_f());
		chdbb.addActionListener(new Chdb());
		showb.addActionListener(new Show());
		writeqb.addActionListener(new Write());
		writemb.addActionListener(new Write2());
		openmb.addActionListener(new Mod() );
		loginb.addActionListener(new ConnectAction() );
		scriptb.addActionListener( new Script());
		opentb.addActionListener(new OpenSaved() );
		helpb.addActionListener(new Help() );
		autob.addActionListener(new Config() );
		showtb.addActionListener(new Showtb());
		exitb.addActionListener(new Exit());
		openb.setToolTipText("Open a query");
		chdbb.setToolTipText("Change current database");
		showtb.setToolTipText("Show relations");
		showb.setToolTipText("Show databases");
		writeqb.setToolTipText("Write a new query");
		writemb.setToolTipText("Write a new modification");
		openmb.setToolTipText("Open a modification");
		loginb.setToolTipText("Log in ");
		scriptb.setToolTipText("Load and execute connection script");
		opentb.setToolTipText("Open a saved table");
		helpb.setToolTipText("Open help on line window");
		autob.setToolTipText("Open Auto configuration window");
		exitb.setToolTipText("Exit");
		jtb.add(loginb);
		jtb.add(scriptb);
		jtb.addSeparator();
		jtb.add(showb);
		jtb.add(chdbb);
		jtb.add(showtb);
		jtb.addSeparator();
		jtb.addSeparator();
		jtb.add(writeqb);
		jtb.add(openb);
		jtb.addSeparator();
		jtb.add(writemb);
		jtb.add(openmb);
		jtb.addSeparator();
		jtb.add(opentb);
		jtb.addSeparator();
		jtb.add(autob);
		jtb.add(helpb);
		jtb.addSeparator();
		jtb.add(exitb);
		contentPane.add(panel);
		contentPane.add(jtb,BorderLayout.NORTH);
		contentPane.add(desktop);
		//Creation of the menu and submenus
		JMenuBar menubar	= new JMenuBar();
		JMenu help			= new JMenu("Help");
		JMenu look			= new JMenu("LookAndFeel!");
		JMenu java			= new JMenu("Java Styles");
		JMenu scriptM		= new JMenu("Script Settings");
		JMenuItem windows	= new JMenuItem("Windows Style",new ImageIcon("Resources/win.gif"));
		JMenuItem def		= new JMenuItem("Java theme",new ImageIcon("Resources/java.gif"));
		JMenuItem motif		= new JMenuItem("Motif Style",new ImageIcon("Resources/motif.gif"));	
		JMenuItem about		= new JMenuItem("About",new ImageIcon("Resources/tip.gif"));
		JMenuItem ruby		= new JMenuItem("Ruby theme",new ImageIcon("Resources/java.gif"));
		JMenuItem aqua		= new JMenuItem("Aqua theme",new ImageIcon("Resources/java.gif"));
		JMenuItem emerald	= new JMenuItem("Emerald theme",new ImageIcon("Resources/java.gif"));	
		JMenuItem charcoal	= new JMenuItem("Charcoal theme",new ImageIcon("Resources/java.gif"));
		JMenuItem contrast	= new JMenuItem("Contrast theme",new ImageIcon("Resources/java.gif"));
		JMenu fileMenu 		= new JMenu("File");
		JMenu options		= new JMenu("Options");
		JMenuItem config	= new JMenuItem("Auto Configuration",new ImageIcon("Resources/gear.gif"));
		JMenuItem show 		= new JMenuItem("Shows Databases",new ImageIcon("Resources/showdata.gif"));
		JMenuItem showTb 	= new JMenuItem("Shows Relations",new ImageIcon("Resources/showdata.gif"));
		JMenuItem query_f 	= new JMenuItem("Open Query",new ImageIcon("Resources/open.gif"));
		JMenuItem openSaved = new JMenuItem("Open Saved Table",new ImageIcon("Resources/open.gif"));
		JMenuItem mod		= new JMenuItem("Open Updein",new ImageIcon("Resources/open.gif"));	 
		JMenuItem login 	= new JMenuItem("Log in",new ImageIcon("Resources/next.gif"));
 	    JMenuItem chdb	 	= new JMenuItem("Change Database",new ImageIcon("Resources/change.gif"));
 	    JMenuItem exit 		= new JMenuItem("Exit",new ImageIcon("Resources/prev.gif"));
	    JMenuItem write 	= new JMenuItem("New Query",new ImageIcon("Resources/palette_draw.gif"));
	    JMenuItem write2 	= new JMenuItem("New Updein",new ImageIcon("Resources/palette_draw.gif"));
	    JMenuItem script 	= new JMenuItem("Load and Execute Connection Script",new ImageIcon("Resources/script.gif"));
	    JMenuItem helpOnLine= new JMenuItem("Help on line",new ImageIcon("Resources/HelpCenter.gif"));
	    help.setMnemonic('H');
	    fileMenu.setMnemonic('F');
	    options.setMnemonic('O');
	    fileMenu.add(login);
	    fileMenu.addSeparator();
	    fileMenu.add(write);
	    fileMenu.add(write2);
	    fileMenu.addSeparator();
	    fileMenu.add(query_f);
	    fileMenu.add(mod);
	   	fileMenu.add(openSaved);
	   	fileMenu.addSeparator();
	    fileMenu.add(script);
	    fileMenu.addSeparator();
	    fileMenu.add(chdb);
	    fileMenu.add(show);
	    fileMenu.add(showTb);
	    fileMenu.addSeparator();
	    fileMenu.addSeparator();
	    fileMenu.add(exit);
	    look.add(windows); 
	    look.add(motif);
	    look.add(java);
	    java.add(def);
	    java.add(ruby);
	    java.add(aqua);
	    java.add(emerald);
	    java.add(charcoal);
	    java.add(contrast);
	    options.add(look);
	    options.add(config);
	    help.add(helpOnLine);
	    help.add(about);
	    menubar.add(fileMenu);
	    menubar.add(options);
	    menubar.add(help);
	    setJMenuBar(menubar);
		//Actions associated with menu voices
		query_f.addActionListener	(new	Query_f());
		openSaved.addActionListener	(new	OpenSaved());
		mod.addActionListener		(new	Mod());
		exit.addActionListener		(new	Exit());
		script.addActionListener	(new	Script());
		show.addActionListener		(new	Show());
		showTb.addActionListener	(new	Showtb());
		login.addActionListener		(new	ConnectAction());
		config.addActionListener	(new	Config());
		chdb.addActionListener		(new	Chdb());
		helpOnLine.addActionListener(new	Help());
		write.addActionListener		(new	Write());
		write2.addActionListener	(new	Write2());
		about.addActionListener		(new 	ActionListener ()
								{
								
									public void actionPerformed(ActionEvent evt)
									{
										JOptionPane.showMessageDialog(null,"System info :\nNumber of processors : "+runtimeStatus.availableProcessors()
										+"\nFree memory in JVM :"+runtimeStatus.freeMemory()/(1024)+" Kb"
										+"\nCurrent connected user : "+user
										+"\n\nMySQLConnectus 0.3\nby Sergio Decherchi\nContact: sedekfx@libero.it\n","Info",1);
									runtimeStatus.gc();
									}
								
								}
								);
		
								
								
		windows.addActionListener ( new ActionListener ()
								{
								
									public void actionPerformed(ActionEvent evt)
									{
									setNewTheme(false,0,windowsStyle);	
									nextSessionIsJavaStyle=false;
									nextSessionJavaTheme=0;
									nextSessionStyle=windowsStyle;
									}
								}
								);
		def.addActionListener ( new ActionListener ()
								{
								
									public void actionPerformed(ActionEvent evt)
									{
									setNewTheme(true,1,javaStyle);			
									nextSessionIsJavaStyle=true;
									nextSessionJavaTheme=1;
									nextSessionStyle=javaStyle;
									
									}
								}
								
								);
		motif.addActionListener ( new ActionListener ()
								{
								
									public void actionPerformed(ActionEvent evt)
									{
									setNewTheme(false,0,motifStyle);					
									nextSessionIsJavaStyle=false;
									nextSessionJavaTheme=0;
									nextSessionStyle=motifStyle;
									
									}
								}
								
								);
		ruby.addActionListener ( new ActionListener ()
								{
								
									public void actionPerformed(ActionEvent evt)
									{
									setNewTheme(true,2,javaStyle);					
									nextSessionIsJavaStyle=true;
									nextSessionJavaTheme=2;
									nextSessionStyle=javaStyle;
									
									}
								}
								
								);
		aqua.addActionListener ( new ActionListener ()
								{
								
									public void actionPerformed(ActionEvent evt)
									{
									setNewTheme(true,6,javaStyle);						
									nextSessionIsJavaStyle=true;
									nextSessionJavaTheme=6;
									nextSessionStyle=javaStyle;
									
									}
								}
								
								);
		charcoal.addActionListener ( new ActionListener ()
								{
								
									public void actionPerformed(ActionEvent evt)
									{
									setNewTheme(true,4,javaStyle);						
									nextSessionIsJavaStyle=true;
									nextSessionJavaTheme=4;
									nextSessionStyle=javaStyle;
									
									}
								}
								
								);
		contrast.addActionListener ( new ActionListener ()
								{
								
									public void actionPerformed(ActionEvent evt)
									{
									setNewTheme(true,5,javaStyle);					
									nextSessionIsJavaStyle=true;
									nextSessionJavaTheme=5;
									nextSessionStyle=javaStyle;
									
									}
								}
								
								);
		emerald.addActionListener ( new ActionListener ()
								{
								
									public void actionPerformed(ActionEvent evt)
									{
									setNewTheme(true,3,javaStyle);						
									nextSessionIsJavaStyle=true;
									nextSessionJavaTheme=3;
									nextSessionStyle=javaStyle;
									
									}
								}
								
								);
								
		
								
		this.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
			currentAccess.saveSettings();
			System.exit(0);
			}
		}
		);
								
		//This closes the splash after 2s , if splash is absent no action is done
		if (!noSplash)
		{
			try
			{
			Thread.sleep(2000);
			}
			catch (InterruptedException ie)
			{
			JOptionPane.showMessageDialog(null,"Thread not regularly interrupted!","Info",0);																											
			}
		splashScreen.setVisible(false);
	   	}
		
		//Set or not autoconfig settings
		if (currentAccess.useAutoConfig())
		{
		currentAccess.setAutoConfigSettings();
		//System.out.println(currentAccess.style);
		setNewTheme(currentAccess.isJavaStyle,currentAccess.javaTheme,currentAccess.style);
		isAutoConfigEnabled=true;
		//By default nextSession* variables are set like startup values
		nextSessionIsJavaStyle	=currentAccess.isJavaStyle;
			if (styleOk)
			{
			nextSessionJavaTheme	=currentAccess.javaTheme;
			nextSessionStyle		=currentAccess.style;
			}
		}
	    if (!currentAccess.useAutoConfig())
			isAutoConfigEnabled=false;
		runtimeStatus.gc();
		
		//System.out.println(nextSessionJavaTheme);
		}

//The different action listeners used in the program.

/**Create an empty window in which you can insert your query text  */
private class Write implements ActionListener
{
	public void actionPerformed(ActionEvent evt)
	{
		windowTitle="Query text";
		(new TextEditFrame(true)).show();
		runtimeStatus.gc();
	}
}
/**Create an empty window in which you can insert your modification text  */
private class Write2 implements ActionListener
{
	public void actionPerformed(ActionEvent evt)
	{
		windowTitle="Modification text";
		(new TextEditFrame(false)).show();
		runtimeStatus.gc();
	}
}
															
/**Open help on line window */
private class Help implements ActionListener
{
	public void actionPerformed(ActionEvent evt)
	{
		if (hhol!=null)
		{
			hhol.doDefaultCloseAction();
		}
		hhol= new HtmlHelpOnLine();
		runtimeStatus.gc();	 								
	}
}
/**Change db class */								
private class Chdb implements ActionListener
{
	public void actionPerformed(ActionEvent evt)
	{
	if (accessDone)
		{
			String oldDB;
			oldDB=new String (currentDB.toString());
			currentDB=null;
			currentDB=(JOptionPane.showInputDialog(null,"Insert the DB's name ","Info",1));
			if ( (currentDB==null) || (currentAccess.executeUpdein(("USE ").concat(currentDB))==false)  )
				{
					if (currentTree!=null)
						{
							currentTree.doDefaultCloseAction();
						}
					if (currentDB!=null)
						{
						JOptionPane.showMessageDialog(null,currentDB.toUpperCase()+" is not present!","Info",2);
						currentDB=oldDB;
						}
					else 
						currentDB=oldDB;
						
					showImmediatelyRelations=true;	
					currentTree=new Tree();
					currentTree.updateTree();
					
				}
				else
				{
					showImmediatelyRelations=true;
					if (currentTree!=null)
						{
							currentTree.doDefaultCloseAction();
						}
					currentTree=new Tree();
					currentTree.updateTree();
				}	
										
			}
		else
			JOptionPane.showMessageDialog(null,"Not logged in!","Info",2);
									
		runtimeStatus.gc();	 								
	}
								
}

/**Panel for option settings */								 
private class Config implements ActionListener
{
	public void actionPerformed(ActionEvent e)
	{
		new OptionPanel();
	}
}
/**Load a query by a saved one */
private class Query_f implements ActionListener
{
										
	public void actionPerformed(ActionEvent evt)
	{
		JFileChooser chooser=new JFileChooser();
		chooser.setCurrentDirectory(new File("."));
		chooser.addChoosableFileFilter(new QueryFilter());
		chooser.setFileView(new FileIconViewForQuery(new QueryFilter(),new ImageIcon("Resources/query.gif")));
		int r =chooser.showOpenDialog(null);
		if (r==JFileChooser.APPROVE_OPTION)
		{
			windowTitle=chooser.getSelectedFile().getName();
			temp=chooser.getSelectedFile().getAbsolutePath();
			TextEditFrame tef;
			currentAccess.readFile(temp,true,true);
			if (!currentAccess.fnf)
				tef=new TextEditFrame(true,queryText);
		}
		runtimeStatus.gc();
	}
										
										
									
}
/**Open a saved table */
private class OpenSaved implements ActionListener 
{

	public void actionPerformed(ActionEvent evt)
	{
		JFileChooser chooser=new JFileChooser();
		chooser.setCurrentDirectory(new File("."));
		chooser.addChoosableFileFilter(new TabFilter());
		chooser.setFileView(new FileIconViewForTab(new TabFilter(),new ImageIcon("Resources/JTable.gif")));
		int r =chooser.showOpenDialog(null);
		if (r==JFileChooser.APPROVE_OPTION)
		{
			windowTitle=chooser.getSelectedFile().getName();
			temp=chooser.getSelectedFile().getAbsolutePath();
		try
		{
			FileInputStream fis = new FileInputStream(temp);
 			ObjectInputStream ois = new ObjectInputStream(fis);
			PermanentResultSet prs = new PermanentResultSet(temp,ois);
			savedTable=new Table(prs.doc,prs.columnNames,desktop,windowTitle);
		}
		catch(FileNotFoundException fnfe)
		{
			JOptionPane.showMessageDialog(null,"File not found!","Info",2);
		}
		catch (IOException ioe)
		{
			JOptionPane.showMessageDialog(null,"IO Error!","Info",2);
		}
		}
		runtimeStatus.gc();
	}
										
										
									
}	

/**Load a modification file */
private class Mod implements ActionListener
{
	public void actionPerformed(ActionEvent evt)
	{
		JFileChooser chooser=new JFileChooser();
		chooser.setCurrentDirectory(new File("."));
		chooser.addChoosableFileFilter(new ModFilter());
		chooser.setFileView(new FileIconViewForMod(new ModFilter(),new ImageIcon("Resources/mod.gif")));
		int r =chooser.showOpenDialog(null);
		if (r==JFileChooser.APPROVE_OPTION)
			{
				windowTitle=chooser.getSelectedFile().getName();
				temp=chooser.getSelectedFile().getAbsolutePath();
				currentAccess.readFile(temp,true,true);
				if (!currentAccess.fnf)
					(new TextEditFrame(false,queryText)).show();
			}
		runtimeStatus.gc();
	}
									
}

/**Save settings and exit */
private class Exit implements ActionListener
{
	public void actionPerformed(ActionEvent evt)
	{
		currentAccess.saveSettings();
		System.exit(0);
	}
}

/**Load and execute a script file */
private class Script implements ActionListener 
{
	public void actionPerformed(ActionEvent evt)
	{
		JFileChooser chooser=new JFileChooser();
		chooser.setCurrentDirectory(new File("."));
		int r =chooser.showOpenDialog(null);
		if (r==JFileChooser.APPROVE_OPTION)
			{
				temp=chooser.getSelectedFile().getAbsolutePath();
				currentAccess.executeScript(temp);
				if (!scriptSet)
					{
						nextSessionScript=temp;
					}
				else
					{
						Object[] options = { "Yes","No" };
						int choosen=JOptionPane.showOptionDialog(null, "You have previously choosen a Script File for\nthe  next session by Auto Configuration window.\nDo you want this script ovverrides the previously selcted? ", "Script", 
						JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE,
						null, options, options[0]);
						if (choosen==0)
							{
								nextSessionScript=temp;
								scriptSet=false;
							}
					}
												
				}
				runtimeStatus.gc();
										
		}
}

/**Show databases in the current tree */
private class Show implements ActionListener
{
		public void actionPerformed(ActionEvent evt)
		{
		//This odd procedure of closing the current table and
		//then rebuilding it is due to ResultSet compatibility
		//between JTree and Table because they both query the
		//DB to charge their results. 
		//Without closing the table the JVM will try to repaint
		//the table accessing the ResultSet (with getValue() method)
		//that in that moment is closed and an exception will be thrown. 
		boolean flag=false;
		showImmediatelyRelations=false;
										
		if ((tab!=null) && (!tab.internalClosed))
		{
			tab.doDefaultCloseAction();
			tab.internalClosed=false;
			flag=true;
		}
		if (currentTree!=null)
		{
			currentTree.doDefaultCloseAction();
		}
		currentTree=new Tree();
		currentTree.updateTree();
		if ((flag) && (!tab.internalClosed) && (currentQuery!=null) )
		{
			tab=new Table(currentAccess.executeQuery(currentQuery),desktop);
			tab.show();
		}
										
										
		
		}
}

/**Show relations in the current tree */
private class Showtb implements ActionListener
{
	public void actionPerformed(ActionEvent evt)
	{
		//This odd procedure of closing the current table and
		//then rebuilding it is due to ResultSet compatibility
		//between JTree and Table because they both query the
		//DB to charge their results. 
		//Without closing the table the JVM will try to repaint
		//the table accessing the ResultSet (with getValue() method)
		//that in that moment is closed and an exception will be thrown. 
		boolean flag=false;	
		if ((tab!=null) && (!tab.internalClosed))
		{
			tab.doDefaultCloseAction();
			//System.out.println(tab.internalClosed);
			tab.internalClosed=false;
			//System.out.println(tab.internalClosed);
			flag=true;
		}
		if (currentTree!=null)
		{
			currentTree.doDefaultCloseAction();
		}
		currentTree = new Tree();
		showImmediatelyRelations=true;
		currentTree.updateTree(); 
		//System.out.println("flag: "+flag+" isClosed: "+tab.internalClosed);
		//System.out.println(currentQuery);
		if ((flag) && (!tab.internalClosed) && (currentQuery!=null))
		{
			tab=new Table(currentAccess.executeQuery(currentQuery),desktop);
			tab.show();
		}
									
	}
}
//Two file filters and IconViews

/**File filter for query*/

private class QueryFilter extends FileFilter
{
	public boolean accept(File f)
	{
	return f.getName().toLowerCase().endsWith(".qry") || f.isDirectory();
	}
	public String getDescription()
	{
	return "Query file";
	}
}
/**Icon View for query*/
private class FileIconViewForQuery extends FileView
{
	private FileFilter myf;
	private Icon myi;
	
	FileIconViewForQuery(FileFilter myfilter,Icon myicon)
	{
		myf=myfilter;
		myi=myicon;
	}
	public Icon getIcon(File f)
	{
		if (!f.isDirectory() && myf.accept(f)  )
		return myi;
		else
		return null;
	}

} 	
/**File filter for modifications file*/
private class ModFilter extends FileFilter
{
	public boolean accept(File f)
	{
	return f.getName().toLowerCase().endsWith(".mod") || f.isDirectory();
	}
	public String getDescription()
	{
	return "Modification file";
	}
}
/**IconView for mod file*/
private class FileIconViewForMod extends FileView
{
	private FileFilter myf;
	private Icon myi;
	
	FileIconViewForMod(FileFilter myfilter,Icon myicon)
	{
		myf=myfilter;
		myi=myicon;
	}
	public Icon getIcon(File f)
	{
		if (!f.isDirectory() && myf.accept(f)  )
		return myi;
		else
		return null;
	}

}
								
										

/**Tree rapresenting database structure */
private class Tree  extends JInternalFrame
{

JTree tree ;
JPanel treePanel;
IconNode root;


	Tree()
	{
	super("Dbms tree");
	setSize(200,500);
	setLocation(0,0);
	
	}

public void updateTree()
{
	
ResultSet rs2=null;
String str="";


									
try{
	rs2=currentAccess.executeQuery("SHOW DATABASES");
	while(rs2.next())
	{
		str=str.concat(" ");
		str=str.concat(rs2.getString(1));
		
	}	
	}
	catch(SQLException sqlex)
	{
		JOptionPane.showMessageDialog(null,"Error in reading DBs","Info",0);								;
		return;
	}	

treePanel= new JPanel(); 
root = new IconNode(user+"@MySQL","Resources/root.gif");
StringTokenizer st = new StringTokenizer(str);
while(st.hasMoreTokens())
root.add(new IconNode(st.nextToken(),"Resources/node.gif"));
tree = new JTree(root);
tree.setCellRenderer(new TreeIcon());

//Invoked by Windows

MouseListener ml = new MouseAdapter() {
     public void mousePressed(MouseEvent e) 
     {
       if (e.getClickCount() == 2) 
       {
		//System.out.println(tree);
		currentQuery=null;
		exploreDb(e);
		showImmediatelyRelations=true;
		}
     }
};

 

//Invoked by Linux

MouseListener m2 = new MouseAdapter() {
     public void mouseReleased(MouseEvent e) 
     {
		if (e.getClickCount() == 2) 
       {
		//System.out.println(tree);
		currentQuery=null;
		exploreDb(e);
		showImmediatelyRelations=true;
              
        }
      } 
  };

tree.addMouseListener(ml);
tree.addMouseListener(m2);
treePanel.add(tree,BorderLayout.CENTER);


JScrollPane jsp = new JScrollPane(treePanel);
jsp.setVisible(false);
getContentPane().add(jsp, BorderLayout.CENTER); 		
jsp.setVisible(true);

	this.setClosable(true);
	this.setMaximizable(true);
	this.setIconifiable(true);
	this.setResizable(true);
	desktop.add(this,1);
	this.show();
	
//System.out.println(showImmediatelyRelations);												

if (showImmediatelyRelations)
{
	rs2=null;
	str="";
									
				try
				{
					rs2=currentAccess.executeQuery("SHOW TABLES");
				
				while(rs2.next())
					{
					str=str.concat(" ");
					str=str.concat(rs2.getString(1));
									
					}	
				}
				catch(SQLException sqlex)
				{
					JOptionPane.showMessageDialog(null,"Error in reading relations","Info",0);								;
					return;
				}
				catch(NullPointerException npe)
				{
				}
					
					IconNode dmtn=root;
					
					st = new StringTokenizer(str);
					
					int i=0;
					if (currentDB!=null && dmtn!=null)
					{
					
					while ( (!(dmtn.toString()).equals(currentDB)) )
					{
					dmtn=(IconNode)dmtn.getNextNode();
					i++;
					if (dmtn==null)
						break;
					//System.out.println(dmtn+" "+currentDB);
					}
					
					}
					
					
					while(st.hasMoreTokens())
					{
					
					if ((temp=st.nextToken())==null || dmtn==null)
						break;
					dmtn.add(new IconNode(temp,"Resources/JTable.gif"));
					}
					
					
					tree.expandRow(i);
					
             
    showImmediatelyRelations=false;
         }
	
}
public void exploreDb(MouseEvent e)
{
//System.out.println(tree+" "+e.getX()+" "+e.getY());
int selRow = tree.getRowForLocation(e.getX(), e.getY());
TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
         if(selRow != -1) {
             
             	if (accessDone)
					{
					if (currentAccess.executeUpdein(("USE ").concat(selPath.getLastPathComponent().toString()))==false)
						{
						if(tab!=null)
						{
							tab.doDefaultCloseAction();
						}
						tab=new Table((currentAccess.executeQuery("SELECT * FROM "+selPath.getLastPathComponent().toString())),desktop);
						tab.show();
						currentQuery="SELECT * FROM "+selPath.getLastPathComponent().toString();
						return;
						}
						
					}
				else
				JOptionPane.showMessageDialog(null,"Not logged in!","Info",2);
				
				runtimeStatus.gc();	 								
				
				ResultSet rs2=null;
				String str="";
									
				try
				{
					rs2=currentAccess.executeQuery("SHOW TABLES");
				while(rs2.next())
					{
					str=str.concat(" ");
					str=str.concat(rs2.getString(1));
									
					}	
				}
				catch(SQLException sqlex)
				{
					JOptionPane.showMessageDialog(null,"Error in reading relations","Info",0);								;
					return;
				}
					
					IconNode dmtn=root;
					
					StringTokenizer st = new StringTokenizer(str);
					//System.out.println(str);
					int index=1;
					
					while (dmtn!=null)
					{
					if (index!=1)
					{
					tree.collapseRow(index-1);
					dmtn.removeAllChildren();
					}
					dmtn=(IconNode)dmtn.getNextNode();
					index++;
					}
					dmtn=root;
					tree.collapseRow(index);
					index=1;
					
					while (!(dmtn.toString()).equals(selPath.getLastPathComponent().toString()))
					{
					dmtn=(IconNode)dmtn.getNextNode();
					index++;
					}
					
					
					
					while(st.hasMoreTokens())
					{
						dmtn.add(new IconNode(st.nextToken(),"Resources/JTable.gif"));
					}
					
					tree.expandRow(index-1);
					//System.out.println("Row : "+index);
					currentDB=selPath.getLastPathComponent().toString();
             		
         }
}

}


 
private void setNewTheme(boolean isJavaStyle,int javaTheme,String style)
{
	try 
	{
	if (isJavaStyle)
	{
	if (javaTheme==1)
	theme = new DefaultMetalTheme();
	else if (javaTheme==2)
	theme=new RubyTheme();
	else if (javaTheme==3)
	theme=new EmeraldTheme();
	else if (javaTheme==4)
	theme=new CharcoalTheme();
	else if (javaTheme==5)
	theme=new ContrastTheme();
	else if (javaTheme==6)
	theme=new AquaTheme();
	
	MetalLookAndFeel.setCurrentTheme(theme);
	}
	
	
	UIManager.setLookAndFeel(style);
	SwingUtilities.updateComponentTreeUI(MainFrame.this);
	}
	catch(ClassNotFoundException cnfe)
	{
	JOptionPane.showMessageDialog(null,"This kind of style is not present in your Java version","Info",0);																											
	styleOk=false;
	nextSessionStyle=javaStyle;
	}
	catch(Exception e)
	{
	JOptionPane.showMessageDialog(null,"This kind of style is not present in your Java version","Info",0);																																																						
	styleOk=false;
	nextSessionStyle=javaStyle;
	}
	runtimeStatus.gc();
}
    




/**Class that manages the help on line window*/

										
 
private class HtmlHelpOnLine extends JInternalFrame  {

    JEditorPane html;
    
    
    HtmlHelpOnLine() {
        
       java.net.URL url 	= 	null;
	   String path			=	null;		
	   
       try {
	   	
		//For linux
		//path ="file:/home/MySQLConnectus/Docs/HelpOnLine/Index.htm";
		//For windows
		path="file:/MySQLConnectus/Docs/HelpOnLine/Index.htm";
		url=new java.net.URL(path);
        html = new JEditorPane(url);
        html.setEditable(false);
        html.addHyperlinkListener(createHyperLinkListener());
		
		JScrollPane scroller = new JScrollPane();
		JViewport vp = scroller.getViewport();
		vp.add(html);
		
        setTitle("Help On Line");
		
		setLocation(0,0);
	
		Toolkit kit 		= Toolkit.getDefaultToolkit();
		Dimension screenSize= kit.getScreenSize();
		int screenWidth 	=((screenSize.width)-150);
		int screenHeight	=((screenSize.height)-150);
		setSize(screenWidth,screenHeight);
		
		Container contentPane= getContentPane();
		contentPane.add(scroller,BorderLayout.CENTER);
		
		this.setClosable(true);
		this.setMaximizable(true);
		this.setIconifiable(true);
		this.setResizable(true);
	
		desktop.add(this,1);
		this.show();											

            
        } 
        catch (MalformedURLException e1) {
            JOptionPane.showMessageDialog(null,"Malformed URL Exception ! " ,"Info",0);																											
        } 
        catch (IOException e) 
        {
           
            JOptionPane.showMessageDialog(null,"File not found! " ,"Info",2);																											
        
        	
        }
    }


//This allows using hiperlink on help on line page
	public HyperlinkListener createHyperLinkListener() {
	return new HyperlinkListener() {
	    public void hyperlinkUpdate(HyperlinkEvent e) {
		if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
		    if (e instanceof HTMLFrameHyperlinkEvent) {
			((HTMLDocument)html.getDocument()).processHTMLFrameHyperlinkEvent(
			    (HTMLFrameHyperlinkEvent)e);
		    } else {
			try {
			    html.setPage(e.getURL());
			} catch (IOException ioe) {
			JOptionPane.showMessageDialog(null,"I/O exception","Info",0);																											    
			}
		    }
		}
	    }
	};
    }
    
}

/**Class that manages the connection step*/

private class ConnectAction implements ActionListener
{
	public void actionPerformed(ActionEvent event)
	{
	if (dialog==null)
	dialog = new PasswordChooser();
	dialog.setUser(new User("user_name",null));
	if (dialog.showDialog(MainFrame.this,"Connect"))
		{
			user=(dialog.getUser()).getName();
			password=(dialog.getUser()).getPassword();
			String ex=currentAccess.access(user,password);
			JOptionPane.showMessageDialog(null,ex,"Info",1);
			if(ex.equals("Connected with success!"))
			{
			accessDone=true;
			currentDB=(JOptionPane.showInputDialog(null,"Insert the DB's name ","Info",1));
			
			//System.out.println("<"+currentDB+">");
			if ( (currentDB!=null) && (!currentDB.equals("")) )
			{
			if (currentAccess.executeUpdein(("USE ").concat(currentDB))==true)
			{
			showImmediatelyRelations=true;
			currentTree.updateTree();
			}
			else
			{
			showImmediatelyRelations=false;
			currentTree.updateTree();
			JOptionPane.showMessageDialog(null,"Database not present!","Info",2);
			}
			
			}
			else
			{
			showImmediatelyRelations=false;
			currentTree.updateTree();
			
			}
			
			}
			
		}
	}
	

}

/**Class that manages the choosing of the password and username*/

private class PasswordChooser extends JPanel
	{
private	JTextField 		username;
private	JPasswordField 	pass;
		JButton 		okButton;
		boolean 		ok;
private	JDialog 		dialog;
	
	PasswordChooser (){
	
		setLayout(new BorderLayout());
		JPanel panel2 = new JPanel();
		panel2.setLayout(new GridLayout(4,4));
		panel2.add(new JLabel("User name : "));
		panel2.add(username= new JTextField(""));
		panel2.add(new JLabel("Password: "));
		panel2.add(pass= new JPasswordField(""));
		add(panel2,BorderLayout.CENTER);
		okButton = new JButton("OK");
		okButton.addActionListener (new ActionListener()
				{
					public void actionPerformed(ActionEvent event)
					{
					ok=true;
					dialog.setVisible(false);
					
					}
				});
		JButton cancelButton = new JButton ("Cancel");
		cancelButton.addActionListener(new ActionListener() 
		{
			public void actionPerformed (ActionEvent event)
			{
				dialog.setVisible(false);
			}
		}
		);
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);
		add(buttonPanel,BorderLayout.SOUTH);
			}
				
	public void setUser(User u)
		{
			username.setText(u.getName());
		}
		
	public User getUser()
		{
			return new User(username.getText(), new String(pass.getPassword())  );
		}
		
	public boolean showDialog(Component parent,String title)
		{
			ok=false;
			Frame owner =null;
			if (parent instanceof Frame)
				owner = (Frame)parent;
			else
				owner =	(Frame)SwingUtilities.getAncestorOfClass(Frame.class,parent);
			if (dialog==null || dialog.getOwner()!=owner)
			{
				dialog=new JDialog(owner,true);
				Container p=dialog.getContentPane();
				p.setLayout(new BoxLayout((dialog.getContentPane()),BoxLayout.Y_AXIS));
				p.add(this);
				dialog.getRootPane().setDefaultButton(okButton);
				dialog.pack();
							
			}		
		dialog.setTitle(title);
		dialog.setSize(200,200);
		dialog.setLocation(150,150);
		
		if (theme!=null)
		{
		MetalLookAndFeel.setCurrentTheme(theme);
		
		}
		SwingUtilities.updateComponentTreeUI(this);
		dialog.show();
		return ok;
		}
		
		
		
		 
	}
private class User
	{
		public User (String aName,String aPassword)	
		{
			name=aName;
			password2=aPassword;
		}
	public String getName() 
	{
	
	return name;
	}
	public String getPassword() 
	{
	
	return password2;
	
	}
	public void setName(String aName){
		name=aName;
	}
	public void setPassword(String aPassword)
	{
	password2=aPassword;
	}
	private String name;
	private String password2;
	
	}

/**This class rapresents the window in which you can insert your custom AuotoConfig settings*/

class OptionPanel extends JInternalFrame
{
	final protected OptionPanel thisOptionPanel=this;
	
	private JRadioButton enableAuto;
	private JRadioButton disableAuto;
	private JRadioButton enableScript;
	private JRadioButton disableScript;
	private JButton setManually;
	private boolean tempIsAutoConfigEnabled;
	private boolean tempNextSessionUseLastLoadedScript;
	private String  tempNextSessionScript;
	private boolean tempScriptSet;
	
	OptionPanel()
	{
	
	setTitle("Auto Configuration Settings");
	setLocation(0,0);
	Container contentPane=getContentPane();
	desktop.add(this,1);
	JPanel pan = new JPanel();
	pan.setBorder(new TitledBorder(null, "Auto Configuration Settings", 
 					 TitledBorder.LEFT, TitledBorder.TOP)); 
	enableAuto =new JRadioButton("Enable AutoConfig");
	disableAuto =new JRadioButton("Disable AutoConfig");
	setManually= new JButton("Set Script");
	
	disableAuto.addItemListener (new ItemListener()
	{
			public void itemStateChanged(ItemEvent e)
			{
			if (((JRadioButton)e.getSource()).isSelected())
			{
				tempIsAutoConfigEnabled=false;
				(thisOptionPanel.enableAuto).setSelected(false);
				(thisOptionPanel.enableScript).setEnabled(false);
				(thisOptionPanel.disableScript).setEnabled(false);
				(thisOptionPanel.setManually).setEnabled(false);
				
			}
			invalidate();
			validate();
			repaint();
			}
	});
	
	enableAuto.addItemListener (new ItemListener() 
	{
			public void itemStateChanged(ItemEvent e)
			{
			if (((JRadioButton)e.getSource()).isSelected())
			{
				tempIsAutoConfigEnabled=true;
				(thisOptionPanel.disableAuto).setSelected(false);
				(thisOptionPanel.enableScript).setEnabled(true);
				(thisOptionPanel.disableScript).setEnabled(true);
				(thisOptionPanel.setManually).setEnabled(true);
				if ((thisOptionPanel.disableScript).isSelected())
					(thisOptionPanel.setManually).setEnabled(false);
			}
			invalidate();
			validate();
			repaint();
			}
	});
	pan.add(enableAuto);
	pan.add(disableAuto);
	JPanel pan2 = new JPanel();
	JPanel pan3 = new JPanel();
	pan2.setBorder(new TitledBorder(null, "Automatic Script Settings", 
 					 TitledBorder.LEFT, TitledBorder.TOP)); 
	enableScript =new JRadioButton("Enable Automatic Script");
	enableScript.addItemListener (new ItemListener()
	{
			public void itemStateChanged(ItemEvent e)
			{
			if (((JRadioButton)e.getSource()).isSelected())
			{
				tempNextSessionUseLastLoadedScript=true;
				(thisOptionPanel.disableScript).setSelected(false);
				(thisOptionPanel.setManually).setEnabled(true);
			}
			invalidate();
			validate();
			repaint();
			}
	});
	disableScript =new JRadioButton("Disable Automatic Script");
	disableScript.addItemListener (new ItemListener()
	{
			public void itemStateChanged(ItemEvent e)
			{
			if (((JRadioButton)e.getSource()).isSelected())
			{
				tempNextSessionUseLastLoadedScript=false;
				tempNextSessionScript=null;
				(thisOptionPanel.enableScript).setSelected(false);
				(thisOptionPanel.setManually).setEnabled(false);
			}
			invalidate();
			validate();
			repaint();
			}
	});
	
	
	
	
	setManually.addActionListener( new ActionListener ()
	{
		public void actionPerformed (ActionEvent evt)
		{
		tempNextSessionUseLastLoadedScript=true;
		JFileChooser chooser=new JFileChooser();
		chooser.setCurrentDirectory(new File("."));
		int r =chooser.showOpenDialog(null);
		if (r==JFileChooser.APPROVE_OPTION)
		{
		tempNextSessionScript=chooser.getSelectedFile().getAbsolutePath();
	
		JOptionPane.showMessageDialog(null,"Next session will start with \nthe selected scripting file.","Info",1);																																																						
		tempScriptSet=true;
		}
																		
        
		} 
	
	});
	JButton ok= new JButton("Ok");
	JButton cancel= new JButton("Cancel");
	ok.addActionListener ( new ActionListener()
	{
		public void actionPerformed(ActionEvent e)
		{
		isAutoConfigEnabled=tempIsAutoConfigEnabled;
		nextSessionUseLastLoadedScript=tempNextSessionUseLastLoadedScript;
		nextSessionScript=tempNextSessionScript;
		scriptSet=tempScriptSet;
		thisOptionPanel.doDefaultCloseAction();
		}
	});
	cancel.addActionListener ( new ActionListener()
	{
		public void actionPerformed(ActionEvent e)
		{
		thisOptionPanel.doDefaultCloseAction();
		}
	});
	
	enableScript.setEnabled(false);
	disableScript.setEnabled(false);
	setManually.setEnabled(false);
	pan2.add(enableScript);
	pan2.add(disableScript);
	pan2.add(setManually);
	pan3.setLocation(50,50);
	pan3.add(ok);
	pan3.add(cancel);
	pan.add(pan2);
	pan.add(pan3);
	
	//Initialization with previous values.
	if (isAutoConfigEnabled)
	{
		enableAuto.setSelected(true);
		disableAuto.setSelected(false);	
		(thisOptionPanel.enableScript).setEnabled(true);
		(thisOptionPanel.disableScript).setEnabled(true);
		(thisOptionPanel.setManually).setEnabled(true);
	
		
		if (nextSessionUseLastLoadedScript)  
		{
		enableScript.setSelected(true);
		disableScript.setSelected(false);
		}
		else 
		{
		enableScript.setSelected(false);
		disableScript.setSelected(true);
		}
	
	}
	else
	{
		enableAuto.setSelected(false);
		disableAuto.setSelected(true);	
	}
	
	contentPane.add(pan);
	
	this.show();											
	
	this.setSize(450,220);
	}
		
}




/**Frame in which you can insert your query or an insert/upadate/delete statement*/
class TextEditFrame extends JInternalFrame implements Printable 
{

		private		boolean 		typ;
		private		JTextArea 		textArea;
		private		int				selectedPage;
		private 	Toolkit			kit;
		private		Dimension		screenSize;
 		private		int				screenWidth;
 		private		int				screenHeight;
 		final 		TextEditFrame 	tef;
		private		JPopupMenu 		popup;
		
	public TextEditFrame(boolean type)
	{

	kit 			=Toolkit.getDefaultToolkit();
	screenSize		=kit.getScreenSize();
	screenWidth 	=((screenSize.width)-50);
	screenHeight	=((screenSize.height)-100);
	
	popup = new JPopupMenu();
	
	JMenuItem cut=new JMenuItem("Cut",new ImageIcon("Resources/cut.gif"));
	JMenuItem paste=new JMenuItem("Paste",new ImageIcon("Resources/paste.gif"));
	JMenuItem printb=new JMenuItem("Print",new ImageIcon("Resources/print.gif"));
	JMenuItem copy=new JMenuItem("Copy",new ImageIcon("Resources/copy.gif"));
	
	//System.out.println(screenWidth+"  "+screenHeight);
	popup.add(cut);
	popup.add(copy);
	popup.add(paste);
	popup.add(printb);
	
	cut.addActionListener(new Cut());
	copy.addActionListener(new Copy());
	paste.addActionListener(new Paste());
	printb.addActionListener(new Print());
	
//This is a tricky variable for referencing the current instance of the TextEditFrame
//usefull when is necessary to print the current text by an inner class like an ActionListener
//linked with the print button. The variable is final cause is accessed by an inner class.
//No necessary if you take the current object by <event>.getSource() method
	
	tef=this;
	selectedPage=0;
	
	setSize(screenWidth/2+80,screenHeight/2); 
	
	typ=type;
	if (typ)
	{
	setTitle(windowTitle);
	
	setLocation(winpos,winpos);
	
	}
	if(!typ)
	{
	setTitle(windowTitle);
	setLocation(winpos+400,winpos);
	}
	
	winpos=winpos+30;
	if (winpos>=150)
	winpos=0;
	
	
	Container contentPane= getContentPane();
	
	this.setClosable(true);
	this.setMaximizable(true);
	this.setIconifiable(true);
	this.setResizable(true);
	
	desktop.add(this,1);
	this.show();											
	
	JPanel panel = new JPanel();
	JButton ex=new JButton("Execute",new ImageIcon("Resources/next.gif"));
	JButton canc = new JButton("Cancel",new ImageIcon("Resources/prev.gif"));
	JButton save = new JButton("Save",new ImageIcon("Resources/save.gif"));
	JButton check = new JButton("Well form",new ImageIcon("Resources/JEditorPane.gif"));
	JButton print = new JButton("Print",new ImageIcon("Resources/print.gif"));
	
	panel.add(check);
	panel.add(print);
	panel.add(ex);
	panel.add(save);
	panel.add(canc);
	
	textArea= new JTextArea(8,40);
	textArea.setLineWrap(true);
	textArea.setWrapStyleWord(true);
	textArea.setFont(new Font("Verdana",0,15));
	
	ex.addActionListener((new Exec() ) );
	canc.addActionListener(new Canc());
	save.addActionListener(new Save());
	check.addActionListener(new Check());
	print.addActionListener( new Print());
	
	contentPane.add(panel,BorderLayout.SOUTH);
	JScrollPane scrollPane = new JScrollPane(textArea);
	
	textArea.addMouseListener(new  RightClick());	
	
	contentPane.add(scrollPane,BorderLayout.CENTER);
	//contentPane.setLayout(new BoxLayout((this.getContentPane()),BoxLayout.X_AXIS));
	textArea.setCaretColor(Color.RED);
	if (type)
	textArea.setText("SELECT <fieldsList>\nFROM <relationsList>\nWHERE <conditionsList>\n");
	else
	textArea.setText("INSERT INTO <relationName>\nVALUES <values>\n\nDELETE FROM <relationName>\nWHERE <conditionsList> \n\nUPDATE <relationName>\nSET <updateList>\nWHERE <conditionsList>");
	
	textArea.setCaretPosition(0);
	
	}
	
	public TextEditFrame(boolean type,String query)
	{
	
	kit 			=Toolkit.getDefaultToolkit();
	screenSize		=kit.getScreenSize();
	screenWidth 	=((screenSize.width)-50);
	screenHeight	=((screenSize.height)-100);
	
	
	popup= new JPopupMenu();
	
	JMenuItem cut=new JMenuItem("Cut",new ImageIcon("Resources/cut.gif"));
	JMenuItem paste=new JMenuItem("Paste",new ImageIcon("Resources/paste.gif"));
	JMenuItem copy=new JMenuItem("Copy",new ImageIcon("Resources/copy.gif"));
	JMenuItem printb=new JMenuItem("Print",new ImageIcon("Resources/print.gif"));
	//System.out.println(screenWidth+"  "+screenHeight);
	popup.add(cut);
	popup.add(copy);
	popup.add(paste);
	popup.add(printb);
	
	cut.addActionListener(new Cut());
	copy.addActionListener(new Copy());
	paste.addActionListener(new Paste());
	printb.addActionListener(new Print());

	tef=this;

	setSize(screenWidth/2+80,screenHeight/2);
	typ=type;
	if (typ)
	{
	setTitle(windowTitle);
	setLocation(winpos,winpos);
	}
	if(!typ)
	{
	setTitle(windowTitle);
	setLocation(winpos+400,winpos);
	}
	
	winpos=winpos+30;
	if (winpos>=150)
	winpos=0;
	
	Container contentPane= getContentPane();
	JPanel panel = new JPanel();
	JButton ex=new JButton("Execute",new ImageIcon("Resources/next.gif"));
	JButton canc = new JButton("Cancel",new ImageIcon("Resources/prev.gif"));
	JButton save = new JButton("Save",new ImageIcon("Resources/save.gif"));
	JButton check = new JButton("Well form",new ImageIcon("Resources/JEditorPane.gif"));
	JButton print = new JButton("Print",new ImageIcon("Resources/print.gif"));
	
	panel.add(check);
	panel.add(print);
	panel.add(ex);
	panel.add(save);
	panel.add(canc);
	
	this.setClosable(true);
	this.setMaximizable(true);
	this.setIconifiable(true);
	this.setResizable(true);

	desktop.add(this,1);
	this.show();											
	
	textArea= new JTextArea(8,40);
	textArea.setLineWrap(true);
	textArea.setWrapStyleWord(true);
	textArea.setFont(new Font("Verdana",0,15));
	
	textArea.addMouseListener(new  RightClick());	
	
	ex.addActionListener((new Exec() ) );
	canc.addActionListener(new Canc());
	save.addActionListener(new Save());
	check.addActionListener(new Check());
	print.addActionListener(new Print());
	
	contentPane.add(panel,BorderLayout.SOUTH);
	JScrollPane scrollPane = new JScrollPane(textArea);
	contentPane.add(scrollPane,BorderLayout.CENTER);
	
	textArea.setText(query);
	textArea.setCaretPosition(0);
	textArea.setCaretColor(Color.RED);
	 
	}
	
	public int print( Graphics g, PageFormat pf, int pageIndex ) {
            
            
            int lastPage=0;
            int printMode=0;
            int i=0;
            int posX=10;
            int posY=10;
            
            
            Graphics2D g2= (Graphics2D)g;
            
            if ( pageIndex == 0 ) {
                
                lastPage = Integer.MAX_VALUE;
               
            }
            if ( printMode == 0 ) {
                if ( pageIndex > 0 )
          		{
          		
                  return NO_SUCH_PAGE;
          
          		}
            }
            else {
                if ( pageIndex > lastPage )
            	{
            	
                  return NO_SUCH_PAGE;
            
            	}
            	}

            
            
           	g2.setFont( new Font("Monotype Corsiva",Font.ITALIC,8) );
           	g2.translate( (int) pf.getImageableX(), (int) pf.getImageableY());
            g2.drawString("Printed with MySQLConnectus 0.3 by SedekFX" , posX, posY );
            
            posY=posY+10;
            
            g2.drawString("",posX,posY);
            g2.setFont( new Font("Verdana",Font.BOLD,9) );
            g2.setColor( Color.black );
			//System.out.println(selectedPage);
            
            int numberOfLines=(((Vector)(orderedData.doc).get(selectedPage)).size());
            
           // System.out.println(numberOfLines);
            for (int k=0;k<numberOfLines;k++)
            {
            g2.drawString((String)( ((Vector)((orderedData.doc).get(selectedPage))).get(k) ), posX, posY );
            posY=posY+10;
            }
            
      		return PAGE_EXISTS;      
            
            
            
			        }
class RightClick extends MouseAdapter
{
	public void mouseClicked(MouseEvent me)
	{
		
	}
	public void mousePressed(MouseEvent me)
	{
			//Invoked by Linux
			if (me.isPopupTrigger())
			popup.show(me.getComponent(),me.getX(),me.getY());
	
	}
	public void mouseReleased(MouseEvent me)
	{
			//Invoked by Windows
			if (me.isPopupTrigger())
			popup.show(me.getComponent(),me.getX(),me.getY());
	
	}
	public void mouseEntered(MouseEvent me)
	{
		
	}
	public void mouseExited(MouseEvent me)
	{
		
	}
}
class Cut implements ActionListener
{
	public void actionPerformed(ActionEvent ae)
	{
	textArea.cut();
	}
}
class Copy implements ActionListener
{
	public void actionPerformed(ActionEvent ae)
	{
	textArea.copy();
	}

}
class Paste implements ActionListener
{
	public void actionPerformed(ActionEvent ae)
	{
	textArea.paste();
	}

}
class Print implements ActionListener
{
	public void actionPerformed(ActionEvent ae)
	{
		
		PageFormat page = null;
		int numPages=0;
		PrinterJob printer = null;
		printer = PrinterJob.getPrinterJob();
        page = printer.defaultPage();
        printer.setPrintable(tef,page);
		
		
		if (textArea.getSelectedText()==null)
		{
			numPages=orderedData.reorderForPrintingOrStreaming(textArea.getText(),65);
		}
		else
			numPages=orderedData.reorderForPrintingOrStreaming(textArea.getSelectedText(),65);
			
		//System.out.println(numPages);
		selectedPage=0;
		try {
        for (int p=0;p<numPages;p++)
        {
            printer.print();
            selectedPage++;
        }
             }
        catch ( Exception e ) {
        JOptionPane.showMessageDialog(null,"No printing device available!","Info",2);																											
        e.printStackTrace();
            				}
	}
}
class Exec implements ActionListener
	{
								
									public void actionPerformed(ActionEvent evt)
									{
									
										if(typ)
										{
											
											//(new Table((currentAccess.executeQuery(textArea.getText()) ),desktop)).show();
											
											if(tab!=null)
											{
											tab.doDefaultCloseAction();
											}
											tab=new Table(currentAccess.executeQuery(textArea.getText()),desktop);
											currentQuery=textArea.getText();
											tab.show();
											
											
										}
										else
										currentAccess.executeUpdein(textArea.getText());
									
									}
								}


class Canc implements ActionListener
	{
								
									public void actionPerformed(ActionEvent evt)
									{
										textArea.setText("");
										textArea.setCaretColor(Color.RED);
										textArea.setCaretPosition(0);
									}
								}
class Check implements ActionListener
	{
								
									public void actionPerformed(ActionEvent evt)
									{
									tef.setLocation(0,0);
									tef.setSize(screenWidth-80,screenHeight-80);
									MiniParser pars=new MiniParser();
									textArea.setText((pars.reorder(textArea.getText())));	
									textArea.setCaretPosition(0);
									textArea.setCaretColor(Color.BLUE);
									
									}
								}

/**This class provides same (now only one!) usefull methods to manipulate a query text*/
class MiniParser
{
	
	MiniParser ()
	{
	
	}

	public String reorder(String text)
	{
		/**This procedure well forms the text
		* Roadmap
		*Implement support for foreign key,primary key
		*improve support for intersect in "from (a) intersect (b)"
		*view , instead of in trigger
		*test 
		*Implement PSM keyword recognition
		*procedure
		*function
		*test
		*support automatic recognition on uncomplete keywords
		*test
		*support very badly formed query
		*final test
		**/
		String 	reorganized=new String();
		temp=null;
		int 	nestingLevel=0;
		boolean subqueryFound		=false;
		boolean perahpsSubqueryEnded=false;
		boolean thereWasDelete		=false;
		boolean triggerDetected		=false;
		boolean perhapsQueryStarted	=false;
		boolean itWasEnd			=false;
		boolean	checkNewLine		=false;
		boolean itWasUEI			=false;
		boolean firstChar			=true;
		boolean isInTable			=false;
		boolean itWasInTable		=false;
		boolean checkDetected		=false; 
		StringTokenizer st = new StringTokenizer(text);
		while (st.hasMoreTokens() /*&& nestingLevel>=0*/)  
		{
    		temp=st.nextToken();
    		//This is neccessary for compatibility between the diffrent keywords
    		if (nestingLevel<0)
    			nestingLevel=0;  
    		// System.out.println(checkDetected);
    		// System.out.println(""+isInTable);
    		
    		if (temp.equalsIgnoreCase("SELECT") )
    		{
    			if (isInTable)
    			{
    				isInTable=false;
    				itWasInTable=true;
    				checkDetected=false;
    			}		
    			if (subqueryFound) 
    			{
    				reorganized=reorganized.concat("SELECT "/*+nestingLevel*/);
    				nestingLevel++;
					subqueryFound=false;	    										    
    				perhapsQueryStarted=false;
    			}
    			else if(perahpsSubqueryEnded)
    			{
    				nestingLevel--;
    				perahpsSubqueryEnded=false;
    				reorganized=reorganized.concat("\n");
    				for(int i=0;i<nestingLevel;i++)
    					reorganized=reorganized.concat(space);
    				reorganized=reorganized.concat("SELECT "/*+nestingLevel*/);
    			}
    			else
    			{
    				reorganized=reorganized.concat("\n");
    				for (int i=0;i<nestingLevel;i++)
    					reorganized=reorganized.concat(space);
    				reorganized=reorganized.concat("SELECT"/*+nestingLevel*/);
    			}
    										        
    		}
    		
    		else if (temp.equalsIgnoreCase("(SELECT")) 
    		{
    			if (isInTable)
    			{
    				isInTable=false;
    				itWasInTable=true;
    				checkDetected=false;
    			}
    			reorganized=reorganized.concat(" ( SELECT ");
    			nestingLevel++;
    			subqueryFound=false;
    		}
    		
    		else if (temp.equalsIgnoreCase("(SELECT*")) 
    		{
    			if (isInTable)
    			{
    				isInTable=false;
    				itWasInTable=true;
    				checkDetected=false;
    			}
    			reorganized=reorganized.concat(" ( SELECT *");
    			nestingLevel++;
    			subqueryFound=false;
    		}
    		
    		else if (temp.endsWith("=(SELECT") || temp.endsWith(",(SELECT")) 
    		{
    			if (isInTable)
    			{
    				isInTable=false;
    				itWasInTable=true;
    				checkDetected=false;
    			}
    			reorganized=reorganized.concat(" "+temp+" ");
    			nestingLevel++;
    			subqueryFound=false;
    		}
    		
    		else if (temp.endsWith("=(select") ) 
    		{
    			if (isInTable)
    			{
    				isInTable=false;
    				itWasInTable=true;
    				checkDetected=false;
    			}
    		int l=(temp.length()-8);
    		reorganized=reorganized.concat(" "+temp.substring(0,l)+" = ( SELECT " );
    		nestingLevel++;
    		subqueryFound=false;
    		}
    		
    		else if (temp.endsWith(",(select")	) 
    		{
    			if (isInTable)
    			{
    				isInTable=false;
    				itWasInTable=true;
    				checkDetected=false;
    			}
    		int l=(temp.length()-8);
    		reorganized=reorganized.concat(" "+temp.substring(0,l)+" , ( SELECT " );
    		nestingLevel++;
    		subqueryFound=false;
    		}
    										     
    		else if (temp.equalsIgnoreCase("FROM"))
    		{
    			if(!thereWasDelete)
    			{
    				reorganized=reorganized.concat("\n");
    				for (int i=0;i<nestingLevel;i++)
    					reorganized=reorganized.concat(space);
    				reorganized=reorganized.concat("FROM");
    			}
    			else
    			{
    				reorganized=reorganized.concat(" FROM ");
    				thereWasDelete=false;
    			}
    		}
    		
    		else if (temp.equalsIgnoreCase("WHERE") )
    		{
    			if (perahpsSubqueryEnded)
    			{
    				nestingLevel--;
    				perahpsSubqueryEnded=false;
    			}
    		reorganized=reorganized.concat("\n");
    		for (int i=0;i<nestingLevel;i++)
    			reorganized=reorganized.concat(space);
    		reorganized=reorganized.concat("WHERE ");
    		}
    		
    		else if (temp.equalsIgnoreCase("GROUP"))
    		{
    			if(perahpsSubqueryEnded)
    			{
    				nestingLevel--;
    				perahpsSubqueryEnded=false;
     			}
    		reorganized=reorganized.concat("\n");
    		for (int i=0;i<nestingLevel;i++)
    			reorganized=reorganized.concat(space);
    		reorganized=reorganized.concat("GROUP");
    		}
    		
    		else if (temp.equalsIgnoreCase("ORDER") )
    		{
    			if(perahpsSubqueryEnded)
    			{
    				nestingLevel--;
    				perahpsSubqueryEnded=false;
     			}
    		reorganized=reorganized.concat("\n");
    		for (int i=0;i<nestingLevel;i++)
    			reorganized=reorganized.concat(space);
    		reorganized=reorganized.concat("ORDER");
    		}
    										     
    		else if (temp.equalsIgnoreCase("UPDATE"))
    		{
    			isInTable=false;
    			if (!triggerDetected)
    			{
    				reorganized=reorganized.concat("\n");
    				for (int i=0;i<nestingLevel;i++)
    					reorganized=reorganized.concat(space);
    				reorganized=reorganized.concat("UPDATE ");
    			}
    			else
    			{
    			reorganized=reorganized.concat(" UPDATE ");
    			triggerDetected=false;
    			}
    		}
    										     
    		else if (temp.equalsIgnoreCase("DELETE"))
    		{
    			isInTable=false;
    			if (!triggerDetected)
    			{
    				reorganized=reorganized.concat("\n");
    				for (int i=0;i<nestingLevel;i++)
    					reorganized=reorganized.concat(space);
    				reorganized=reorganized.concat("DELETE ");
    				thereWasDelete=true;
    			}
    			else
    			{
    				reorganized=reorganized.concat(" DELETE ");
    				triggerDetected=false;
    			}
    		}
    										     
    		else if (temp.equalsIgnoreCase("SET"))
    		{
    			isInTable=false;
    			reorganized=reorganized.concat("\n");
    			for (int i=0;i<nestingLevel;i++)
    				reorganized=reorganized.concat(space);
    			reorganized=reorganized.concat("SET ");
    		}
    										     
    		else if (temp.equalsIgnoreCase("union"))
    		{
    			reorganized=reorganized.concat("\n");
    			for (int i=0;i<nestingLevel;i++)
    				reorganized=reorganized.concat(space);
    			reorganized=reorganized.concat("UNION");
    			itWasUEI=true;
    		}
    										     
    		else if (temp.equalsIgnoreCase("intersect"))
    		{
    			reorganized=reorganized.concat("\n");
    			for (int i=0;i<nestingLevel;i++)
    				reorganized=reorganized.concat(space);
    			reorganized=reorganized.concat("INTERSECT");
    			itWasUEI=true;
    		}
    										     
    		else if (temp.equalsIgnoreCase("except"))
    		{
    			reorganized=reorganized.concat("\n");
    			for (int i=0;i<nestingLevel;i++)
    				reorganized=reorganized.concat(space);
    			reorganized=reorganized.concat("EXCEPT");
    			itWasUEI=true;
    		}
    										     
    		else if (temp.startsWith("WHEN") || temp.startsWith("when"))
    		{
    			reorganized=reorganized.concat("\n");
    			for (int i=0;i<nestingLevel;i++)
    				reorganized=reorganized.concat(space);
    			reorganized=reorganized.concat("WHEN"+temp.substring(4,temp.length()));
    		}
    										     
    		else if (temp.equalsIgnoreCase("INSERT") )
    		{
    			isInTable=false;
    			if (!triggerDetected)
    			{
    				reorganized=reorganized.concat("\n");
    				for (int i=0;i<nestingLevel;i++)
    					reorganized=reorganized.concat(space);
    				reorganized=reorganized.concat("\nINSERT");
    			} 
    			else
    			{
    				reorganized=reorganized.concat(" INSERT ");
    				triggerDetected=false;
    				nestingLevel=0;
    			}
    		}
    										     
    		else if (temp.equalsIgnoreCase("VALUES"))
    		{
    			reorganized=reorganized.concat("\n");
    			for (int i=0;i<nestingLevel;i++)
    				reorganized=reorganized.concat(space);
    			reorganized=reorganized.concat("VALUES");
    		}
    										     
    		else if (temp.equalsIgnoreCase("CREATE"))
    		{
    			isInTable=false;
    			reorganized=reorganized.concat("\n\n");
    			reorganized=reorganized.concat("CREATE ");
    			nestingLevel=0;
    			triggerDetected=false;
    			subqueryFound=false;
				perahpsSubqueryEnded=false;
			}
    										     
    		else if (temp.equalsIgnoreCase("AFTER"))
    		{
    			reorganized=reorganized.concat("\n");
    			reorganized=reorganized.concat("AFTER ");
    			triggerDetected=true;
    		}
    										     
    		else if (temp.equalsIgnoreCase("BEFORE"))
    		{
    			reorganized=reorganized.concat("\n");
    			reorganized=reorganized.concat("BEFORE");
    			triggerDetected=true;
    		}
    										     
    		else if (temp.equalsIgnoreCase("DROP"))
    		{
    			reorganized=reorganized.concat("\n");
    			for (int i=0;i<nestingLevel;i++)
    				reorganized=reorganized.concat(space);
    			reorganized=reorganized.concat("DROP");
    		}
    										     
    		else if (temp.equalsIgnoreCase("ADD"))
    		{
    			reorganized=reorganized.concat("\n");
    			for (int i=0;i<nestingLevel;i++)
    				reorganized=reorganized.concat(space);
    			reorganized=reorganized.concat("ADD");
    		}
    										     
    		else if (temp.equalsIgnoreCase("having"))
    		{
    			reorganized=reorganized.concat("\n");
    			for (int i=0;i<nestingLevel;i++)
    				reorganized=reorganized.concat(space);
    			reorganized=reorganized.concat("HAVING");
    		}
    										     
    		else if (temp.equalsIgnoreCase("table"))
    		{
    			reorganized=reorganized.concat(" TABLE");
    			isInTable=true;
    		}
    											     
    		else if (temp.equalsIgnoreCase("assertion"))
    		{
    			reorganized=reorganized.concat(" ASSERTION");
    			checkNewLine=true;
    		}
    										     
    		else if (temp.equalsIgnoreCase("BY") )
    			reorganized=reorganized.concat(" BY");
    										     
    		else if (temp.equalsIgnoreCase("REFERENCING") )
    			reorganized=reorganized.concat("\nREFERENCING");
    										     
    		else if (temp.equalsIgnoreCase("on"))
    			reorganized=reorganized.concat(" ON");
    										     
    		else if (temp.equalsIgnoreCase("new"))
    			reorganized=reorganized.concat("\nNEW");
    										     
    		else if (temp.equalsIgnoreCase("old"))
    			reorganized=reorganized.concat("\nOLD");
    										     
    		else if (temp.equalsIgnoreCase("into"))
    			reorganized=reorganized.concat(" INTO ");
    										     
    		else if (temp.equalsIgnoreCase("for"))
    			reorganized=reorganized.concat("\nFOR ");
    										     
    		else if (temp.equalsIgnoreCase("each"))
    			reorganized=reorganized.concat(" EACH ");
    										     
    		else if (temp.equalsIgnoreCase("begin"))
    		{
    			reorganized=reorganized.concat("\nBEGIN ");
    			nestingLevel=0;
    			subqueryFound=false;	    										    
    			perhapsQueryStarted=false;
    		}
    										     
    		else if (temp.startsWith("end") || temp.startsWith("END"))
    		{
    			reorganized=reorganized.concat("\nEND "+temp.substring(3,temp.length())+" ");
    			nestingLevel=0;
    			subqueryFound=false;	    										    
    			perhapsQueryStarted=false;
    			itWasEnd=true;	
    		}
    										     
    		else if (temp.equalsIgnoreCase("row"))
    			reorganized=reorganized.concat(" ROW");
    										     
    		else if (temp.equalsIgnoreCase("on"))
    			reorganized=reorganized.concat(" ON");
    										     
    		else if (temp.equalsIgnoreCase("of"))
    			reorganized=reorganized.concat(" OF");
    										     
    		else if (temp.equalsIgnoreCase("out"))
    			reorganized=reorganized.concat(" OUT");
    										     
    		else if (temp.equalsIgnoreCase("inout"))
    			reorganized=reorganized.concat(" INOUT");
    										     
    		else if (temp.equalsIgnoreCase("date"))
    			reorganized=reorganized.concat(" DATE");
    										     
    		else if (temp.equalsIgnoreCase("time"))
    			reorganized=reorganized.concat(" TIME");
    										     
    		else if (temp.equalsIgnoreCase("unique"))
    			reorganized=reorganized.concat(" UNIQUE");
    										     
    		else if (temp.equalsIgnoreCase("in"))
    		{
    			reorganized=reorganized.concat(" IN");
    			perhapsQueryStarted=true;
    		}
    										     
    		else if (temp.equalsIgnoreCase("join"))
    			reorganized=reorganized.concat(" JOIN");
    										     
    		else if (temp.equalsIgnoreCase("check"))
    		{
    			if (!checkNewLine)
    			{
    				reorganized=reorganized.concat(" CHECK");
    				checkDetected=true;
    			}
    			else
    			{
    				checkNewLine=false;
    				reorganized=reorganized.concat("\nCHECK");
    			}
    		}
    										     
    		else if (temp.equalsIgnoreCase("exists"))
    			reorganized=reorganized.concat(" EXISTS");
    										    
    		else if (temp.equalsIgnoreCase("natural"))
    			reorganized=reorganized.concat(" NATURAL");
    										     
    		else if (temp.equalsIgnoreCase("cross"))
    			reorganized=reorganized.concat(" CROSS");
    										     
    		else if (temp.equalsIgnoreCase("outer"))
    			reorganized=reorganized.concat(" OUTER");
    										     
    		else if (temp.endsWith("all") || temp.endsWith("ALL") )
    			reorganized=reorganized.concat(temp.substring(0,temp.length()-3)+" ALL");
    										     
    		else if (temp.endsWith("any") || temp.endsWith("ANY") )
    			reorganized=reorganized.concat(temp.substring(0,temp.length()-3)+" ANY");
    										     
    		else if (temp.equalsIgnoreCase("as"))
    		{
    			reorganized=reorganized.concat(" AS ");
    			if(perahpsSubqueryEnded)
    			{
    				nestingLevel--;
    				perahpsSubqueryEnded=false; 
    				//reorganized=reorganized.concat(" <IO> ");
    			}
    		}
    										     
    		else if (temp.equalsIgnoreCase("and"))
    			reorganized=reorganized.concat(" AND");
    										     
    		else if (temp.equalsIgnoreCase("distinct"))
    			reorganized=reorganized.concat(" DISTINCT");
    										     
    		//Put like startsWith all agregations
    		else if (temp.equalsIgnoreCase("sum"))
    			reorganized=reorganized.concat(" SUM");
    										     
    		else if (temp.equalsIgnoreCase("sum(*)") ) 
    			reorganized=reorganized.concat(" SUM(*)");
    										     
    		else if (temp.equalsIgnoreCase("avg"))
    			reorganized=reorganized.concat(" AVG");
    										     
    		else if (temp.equalsIgnoreCase("avg(*)"))
    			reorganized=reorganized.concat(" AVG(*)");
    										     
    		else if (temp.equalsIgnoreCase("count"))
    			reorganized=reorganized.concat(" COUNT");
    										     
    		else if (temp.equalsIgnoreCase("count(*)"))
    			reorganized=reorganized.concat(" COUNT(*)");
    										     
    		else if (temp.equalsIgnoreCase("min"))
    			reorganized=reorganized.concat(" MIN");
    										     
    		else if (temp.equalsIgnoreCase("min(*)"))
    			reorganized=reorganized.concat(" MIN(*)");
    										     
    		else if (temp.equalsIgnoreCase("max"))
    			reorganized=reorganized.concat(" MAX");
    										     
    		else if (temp.equalsIgnoreCase("max(*)"))
    			reorganized=reorganized.concat(" MAX(*)");
    										     
    		else if (temp.equalsIgnoreCase("left"))
    			reorganized=reorganized.concat(" LEFT");
    										     
    		else if (temp.equalsIgnoreCase("right"))
    			reorganized=reorganized.concat(" RIGHT");
    										     
    			else if (temp.equalsIgnoreCase("full"))
    				reorganized=reorganized.concat(" FULL");
    										     
    			else if (temp.equalsIgnoreCase("cascade"))
    				reorganized=reorganized.concat(" CASCADE");
    										     
    			else if (temp.equalsIgnoreCase("null"))
    				reorganized=reorganized.concat(" NULL");
    										     
    			else if (temp.equalsIgnoreCase("default"))
    				reorganized=reorganized.concat(" DEFAULT");
    										     
    			else if (temp.equalsIgnoreCase("or"))
    				reorganized=reorganized.concat(" OR");
    										     
    			else if (temp.equalsIgnoreCase("not"))
    				reorganized=reorganized.concat(" NOT");
    										     
    			else if (temp.equalsIgnoreCase("like"))
    				reorganized=reorganized.concat(" LIKE");
    										     
    			else if (temp.startsWith("char") || temp.startsWith("CHAR"))
    				reorganized=reorganized.concat(" CHAR"+temp.substring(4,temp.length()));
    										     
    			else if (temp.startsWith("varchar") || temp.startsWith("VARCHAR"))
    				reorganized=reorganized.concat(" VARCHAR"+temp.substring(7,temp.length()));
    										     
    			else if (temp.equalsIgnoreCase("integer"))
    				reorganized=reorganized.concat(" INTEGER");
    										     
    			else if (temp.equalsIgnoreCase("float"))
    				reorganized=reorganized.concat(" FLOAT");
    										     
    			else if (temp.equalsIgnoreCase("trigger"))
    				reorganized=reorganized.concat(" TRIGGER");
    										     
    			else if (temp.equalsIgnoreCase("("))
    			{
    				if(!firstChar)
    				{
    					if (!itWasUEI)
    					{
    						reorganized=reorganized.concat(" ( ");
    						subqueryFound=true;
    						if (perhapsQueryStarted)
    							subqueryFound=true;
    						//reorganized=reorganized.concat((new Boolean(subqueryFound)).toString());
    					}
    					else
    					{
    						reorganized=reorganized.concat("\n");	
    						for (int i=0;i<nestingLevel;i++)
    							reorganized=reorganized.concat(space);
    						reorganized=reorganized.concat(" ( ");
    						itWasUEI=false;
    						subqueryFound=false;
    						perahpsSubqueryEnded=false;
    					}
    				}
    				else
    					reorganized=reorganized.concat("\n( ");
    			}
    										     
    			else if (temp.endsWith(")WHERE")||temp.endsWith(")where"))
    			{
    				reorganized=reorganized.concat(" "+temp.substring(0,temp.length()-6)+" )");
    				nestingLevel--;
    			}
    			
    			else if (temp.endsWith(")"))
    			{
    				if (!itWasEnd)
    				{
    					if (itWasInTable)
    					{
    						itWasInTable=false;
    						isInTable=true;
    					}
    										     
    				//reorganized=reorganized.concat("<" + nestingLevel+ ">" );
    				reorganized=reorganized.concat(" "+temp.substring(0,temp.length()-1)+" ) ");
    				if ((perahpsSubqueryEnded) && (nestingLevel>0))
    				{
    					nestingLevel--;
    				}
    				perahpsSubqueryEnded=true;
    				subqueryFound=false;
    				}
    				else
    				{
    					reorganized=reorganized.concat(" "+temp.substring(0,temp.length()-1)+" ) ");
    					perahpsSubqueryEnded=false;
    					subqueryFound=false;
    					itWasEnd=false;
    				}
    			}
    										        
    			else
    			{
    				if (isInTable)
    				{
    					if (checkDetected)
						{
							reorganized=reorganized.concat(" "+temp);
							//System.out.println("%%");
							checkDetected=false;
						}	
	    				else
	    					reorganized=reorganized.concat("\n"+temp);
    				}	
    				else
    					reorganized=reorganized.concat(" "+temp);
    			}
    										     
    								
    			firstChar=false;
   				//System.out.println(checkDetected);
   		}
   									
 									
 	/*nestingLevel=0;
	subqueryFound=false;
	perahpsSubqueryEnded=false;
	isInTable=false;
	itWasInTable=false;*/
	return reorganized;
	}
}

class Save implements ActionListener
	{
								
									public void actionPerformed(ActionEvent evt)
									{
									JFileChooser chooser=new JFileChooser();
									
											chooser.setCurrentDirectory(new File("."));
											if (typ)
											{
											chooser.addChoosableFileFilter(new QueryFilter());
											chooser.setFileView(new FileIconViewForQuery(new QueryFilter(),new ImageIcon("Resources/query.gif")));
											}
											else
											{
											chooser.addChoosableFileFilter(new ModFilter());
											chooser.setFileView(new FileIconViewForMod(new ModFilter(),new ImageIcon("Resources/mod.gif")));
											}
											
											
											int r =chooser.showSaveDialog(null);
											
											if (r==JFileChooser.APPROVE_OPTION)
											{
												windowTitle=chooser.getSelectedFile().getName();
												temp=chooser.getSelectedFile().getAbsolutePath();
												if ((typ) && (!temp.endsWith(".qry"))) 
												{
													temp=temp.concat(".qry");
													windowTitle=windowTitle.concat(".qry");
												}
												
												if (!typ)
												{
													if (!temp.endsWith(".mod"))
													{
													temp=temp.concat(".mod");
													windowTitle=windowTitle.concat(".mod");
													}
												}
												
												
												if(currentAccess.saveFile(textArea.getText(),temp))
												JOptionPane.showMessageDialog(null,"File saved as "+windowTitle,"Info",1);																			
												else
												JOptionPane.showMessageDialog(null,"Error during file writing  ","Info",0);																											
												
												tef.setTitle(windowTitle);
											}	
										
									}
								}


}	



/**Core class of the program that manages the access to the DBMS, read/write files tools and other services*/

private class Axxess { 
    
     	 
private      	Connection conn			=	null;
private 	 	Statement 	stmt 		= 	null; 
private			ResultSet 	rs 			= 	null; 
private		 	BufferedReader stdin	=	null;
private         String temp				=	null;
private         String exitusStatement	=	null;	
public			boolean fnf				= 	false;
//Variables for autoconfig feature
public			boolean isJavaStyle		=	false;         
public			int javaTheme			=	0;
public			String style			=	"";


Axxess()
    {
    	exitusStatement="Connected with success!";	
    }         
        
        	
public String access(String us,String pass) {

		exitusStatement="Connected with success!";
		temp=null;
		
		try { 
            
            Class.forName("com.mysql.jdbc.Driver").newInstance(); 
        	
        	} 
        	catch (Exception ex) 
        	{ 
            JOptionPane.showMessageDialog(null,"\nClass initialization error ''com.mysql.jdbc.Driver'' \ncontrol your classpath","Info",2);
            System.exit(1); 
        	}
		
		temp="jdbc:mysql://localhost/";
        
       
				temp=temp.concat("?user=");
				temp=temp.concat(us);	
				temp=temp.concat("&password=");
				temp=temp.concat(pass);	
				
		try {
            
            conn = DriverManager.getConnection(temp);
          
            } 
        	
        catch (SQLException ex) 
        	
        	{
            JOptionPane.showMessageDialog(null,"Your sql server is not activated/installed or password wrong","Info",0);
        	return exitusStatement="Connection failed!";
        	}

			
		try {
    
    		stmt = conn.createStatement(); 

    		}
    	catch (SQLException ex2)
    		{
    		JOptionPane.showMessageDialog(null,"SQLException: " + ex2.getMessage()+"\nSQLState: " + ex2.getSQLState()+"\nVendorError: " + ex2.getErrorCode(),"Info",2);
    		return exitusStatement="Connection failed!";
        	}
   // System.out.println("update");    	
if (currentTree!=null)
{
 currentTree.doDefaultCloseAction();
}
currentTree=new Tree();


	return exitusStatement;    
			}

public ResultSet readFile(String filename,boolean ff,boolean exit)
{
	//String temp="";
	BufferedReader br = null;
	String tmp=null;
	
	fnf=false;
	
	
	try
	{
		br = new BufferedReader(new FileReader(filename));
		temp="";
		while((tmp=br.readLine()) != null)
		{
			 temp=temp.concat(tmp);
			 temp=temp.concat("\n ");	
			 
		}
	queryText=temp;
	}	
	catch(FileNotFoundException fnfe)
	{
		JOptionPane.showMessageDialog(null,"File <"+filename+"> not present!","Info",0);
		fnf=true;
		return null;
	}
	catch(IOException ioe)
	{
		ioe.printStackTrace();
		JOptionPane.showMessageDialog(null,"File reading error!","Info",0);
		fnf=true;
		return null;
	}
	finally
	{
		if(br != null)
		{
			try
			{
			br.close();
			}
			catch(IOException ioe)
			{
			ioe.printStackTrace();
			}
		}
	}
	if(exit)
	return null;
	if (ff)
	return executeQuery(temp);
	if(!ff)
	executeUpdein(temp);
					
return null;
}



public void executeScript(String filename)
{

user=null;
password=null;

String dbname=null;

int i=0;
temp=null;

//System.out.println(filename);
if((this.readFile(filename,true,true)==null) && (fnf))
{
JOptionPane.showMessageDialog(null,"Script file not loaded!","Info",0);
return;
}

StringTokenizer st = new StringTokenizer(queryText);

while(st.hasMoreTokens())
{
temp=st.nextToken();
//System.out.println(temp);
if(i==0)
{
//System.out.println((temp.substring(0,6)));
if ((temp.substring(0,6)).equals("USER->"))
	{
	//System.out.println(temp.substring(6,temp.length()));
	user=temp.substring(6,temp.length());
	}
else 
	{
	JOptionPane.showMessageDialog(null,"Bad formed script file!","Info",0);
	return;
	}
	

}
if (i==1)
{
if ( temp.startsWith("PASSWORD->"))
	{
	if ((temp.substring(10,temp.length())).equals("null"))
	password="";
	else
	password=temp.substring(10,temp.length());
	//System.out.println(password);
	}
	
else
	{
	JOptionPane.showMessageDialog(null,"Bad formed script file!","Info",0);
	return;
	}
	

}
if (i==2)
{
if (temp.startsWith("DBNAME->"))
	{
		dbname=temp.substring(8,temp.length());
		currentDB=dbname;
	}
	
else
	{
	JOptionPane.showMessageDialog(null,"Bad formed script file!","Info",0);
	return;
	}
//System.out.println(dbname);	
//System.out.println(temp.substring(8,temp.length()));
}

i++;

}
if ((user!=null) && (password!=null))
this.access(user,password);
else
{
JOptionPane.showMessageDialog(null,"Invalid fields!","Info",2);
return;
}

if ( (dbname!=null) && (!dbname.equalsIgnoreCase("")) && (!dbname.equalsIgnoreCase(" "))  )
{

	
if (currentTree!=null)
{
if ((this.executeUpdein("USE "+dbname))==false)
{
	showImmediatelyRelations=false;
	JOptionPane.showMessageDialog(null,dbname.toUpperCase()+" is not present!","Info",2);
}
else
showImmediatelyRelations=true;

currentTree = new Tree();
currentTree.updateTree();
}

}

else
{
JOptionPane.showMessageDialog(null,"DB name not specified!","Info",2);
showImmediatelyRelations=false;
currentTree.updateTree();
accessDone=true;
return;
}


accessDone=true;
return;

}
public boolean useAutoConfig()
{

try
{
readFile("config.txt",true,true);
StringTokenizer st = new StringTokenizer(queryText);
//String temp=st.nextToken();
temp=st.nextToken();
if (temp.equalsIgnoreCase("false"))
return false;
if (temp.equalsIgnoreCase("true"))
return true;

}
catch(NullPointerException npe)
{
JOptionPane.showMessageDialog(null,"Invalid config file\ndefault settings applied !","Info",2);
return false;
}

return false;
}

public void setAutoConfigSettings()
{
readFile("config.txt",true,true);
StringTokenizer st = new StringTokenizer(queryText);
int i=0;
//String temp="";
temp="";
st.nextToken();

while (st.hasMoreTokens())
{
temp=st.nextToken();
if (i==0)
{
	if (temp.equalsIgnoreCase("false"))
	isJavaStyle=false;
	else if (temp.equalsIgnoreCase("true"))
	isJavaStyle=true;
	else
	{
	JOptionPane.showMessageDialog(null,"Invalid config file\ndefault settings applied !","Info",2);
	return;
	}
}
if (i==1)
{
	try
	{
	javaTheme=(new Integer(temp)).intValue();
	if (javaTheme>6)
		{
		JOptionPane.showMessageDialog(null,"Invalid config file\ndefault settings applied !","Info",2);
		nextSessionJavaTheme=1;
		return;
		}
	}
	catch(Exception e)
	{
	JOptionPane.showMessageDialog(null,"Invalid config file\ndefault settings applied !","Info",2);
	return;
	}
	
}
if (i==2)
{
	//System.out.println(temp);
	style=new String(temp);
	//System.out.println(style);
}
if (i==4)
{
	String tmp=new String();
	
	//System.out.println(temp);
	if (!temp.endsWith("txt"))
	{
	
	while (!(tmp=st.nextToken()).endsWith("txt"))
	{
	temp=temp.concat(" "+tmp);
	}
	
	temp=temp.concat(" "+tmp);
	}
	nextSessionScript=temp;
	this.executeScript(temp);	
}
if (i==3)
{
	if (temp.equalsIgnoreCase("true"))
	{
	nextSessionUseLastLoadedScript=true;
	}
	
}

i++;
}


}
public boolean saveFile(String text,String name)
{
	
	PrintWriter out=null;
	try 
	{
		out= new PrintWriter(new BufferedWriter(new FileWriter(name))); 
		out.println( text );
		
		if (out!=null)
		{
		out.flush();
		out.close();
		}
		
	}
	catch (IOException ioe) 
	{
		return false;
	}


return true;
}

public void saveSettings()
{
String tmp="";
if (isAutoConfigEnabled)
tmp=new String("true");
if (!isAutoConfigEnabled)
tmp=new String("false");

tmp=tmp.concat(" ");

if (nextSessionIsJavaStyle)
tmp=tmp.concat("true" );
else
tmp=tmp.concat("false" );

tmp=tmp.concat(" ");

tmp=tmp.concat((new Integer(nextSessionJavaTheme)).toString() );

tmp=tmp.concat(" ");

if (nextSessionStyle!=null)
tmp=tmp.concat(nextSessionStyle);
else
tmp=tmp.concat(javaStyle);

tmp=tmp.concat(" "+(new Boolean (nextSessionUseLastLoadedScript)).toString());

tmp=tmp.concat(" ");

if ((nextSessionScript!=null) && (nextSessionUseLastLoadedScript))
tmp=tmp.concat(nextSessionScript);

saveFile(tmp,"config.txt");
}

public boolean executeUpdein(String query)
{

try 
        	{
                     	
    		 try
    		 {
    		 stmt.executeUpdate(query); 
			 }
			 catch (NullPointerException npe)
			 {
			 JOptionPane.showMessageDialog(null,"No DB selected\n or access not done","Info",2);
			 return false;
			 }	
			}		

			catch (SQLException ex)
			{
				return false;		
			}
			
JOptionPane.showMessageDialog(null,"Action executed with success!","Info",1);	 		
return true;

}

public ResultSet executeQuery (String query) 
{
   			try 
        	{
        	
    		rs = stmt.executeQuery(query); 
 			} 
        	catch (SQLException ex)
			{
			JOptionPane.showMessageDialog(null,"SQLException: " + ex.getMessage()+"\nSQLState: " + ex.getSQLState()+"\nVendorError: " + ex.getErrorCode()   ,"Info",2);
			return null;
			}
	 		catch (NullPointerException npe)
	 		{
	 		JOptionPane.showMessageDialog(null,"No DB selected\n or access not done"  ,"Info",2);
			return null;
			
	 		}
	 		
	return rs;
	 		
 }
}
}
	